@@ -53,6 +53,11 @@
 ##' generated by applying the \code{formals} function to the on-the-fly function name and then setting the according 
 ##' parameter one wishes to change.     
 ##' @param onTheFlyOptLogFilePath Path to optional log file for on-the-fly function
+##' @param onTheFlyFunction pass in a function pointer. This function will be called with the path to the
+##' current media file. It is required that the function returns a tibble/data.frame like object that contains
+##' a column called \code{frame_time} that specifies the time point of each row. \code{get_trackdata} will then
+##' extract the rows belonging to the current segment. This allows users to code their own function to be used with
+##' \code{get_trackdata} and allows for most data formats to be used within an emuDB.
 ##' @param resultType Specify class of returned object. Either \code{"emuRtrackdata"}, 
 ##' \code{"trackdata"} or \code{"tibble"} == the default  (see \code{\link{trackdata}}, \code{\link{emuRtrackdata}} 
 ##' and \code{\link{tibble}} for details about these objects).
@@ -104,6 +109,7 @@
                             onTheFlyFunctionName = NULL, 
                             onTheFlyParams = NULL, 
                             onTheFlyOptLogFilePath = NULL, 
+                            onTheFlyFunction = NULL,
                             resultType = "tibble",
                             consistentOutputType = TRUE, 
                             verbose = TRUE){
@@ -114,16 +120,29 @@
   # get DBconfig
   DBconfig = load_DBconfig(emuDBhandle)
   
+  # convert factors into characters
+  if("tbl_df" %in% class(seglist)){
+    seglist = seglist %>% dplyr::mutate_if(is.factor, as.character)
+  }
+  
   #########################
   # parameter checks  
   
-  # set ssffTrackName to fist tracks entry in wrasspOutputInfos if ssffTrackName is not set
+  # set ssffTrackName to first tracks entry in wrasspOutputInfos if ssffTrackName is not set
   if(!is.null(onTheFlyFunctionName) && is.null(ssffTrackName)){
     ssffTrackName = wrassp::wrasspOutputInfos[[onTheFlyFunctionName]]$tracks[1]
   }
   
+  # checks for onTheFlyFunction
+  if(is.function(onTheFlyFunction)){
+    if(resultType != "tibble"){
+      stop('onTheFlyFunction only works with with resultType = "tibble"')
+    }
+    ssffTrackName = "CUSTOM_FUNCTION"
+  }
+  
   # check if all values for minimal call are set
-  if( is.null(emuDBhandle) || is.null(seglist) || is.null(ssffTrackName)) {
+  if(!is.function(onTheFlyFunction) && (is.null(emuDBhandle) || is.null(seglist) || is.null(ssffTrackName))) {
     stop("emuDBhandle, seglist and ssffTrackName have to all be set!\n")
   }
   
@@ -185,6 +204,12 @@
     trackDef[[1]]$name = "MEDIAFILE_SAMPLES"
     trackDef[[1]]$columnName =  "audio"
     trackDef[[1]]$fileExtension = DBconfig$mediafileExtension
+  }else if(ssffTrackName %in% c("CUSTOM_FUNCTION")){
+    trackDef = list()
+    trackDef[[1]] = list()
+    trackDef[[1]]$name = "CUSTOM_FUNCTION"
+    trackDef[[1]]$columnName = "CUSTOM_FUNCTION"
+    trackDef[[1]]$fileExtension = "CUSTOM_FUNCTION"
   }else{
     if(is.null(onTheFlyFunctionName)){
       trackDefFound = sapply(DBconfig$ssffTrackDefinitions, 
@@ -278,8 +303,12 @@
   prevUtt = ""
   bndls = list_bundles(emuDBhandle)
   
-  # loop through bundle names
+  # init result lists & index
   curIndexStart = 1
+  data_list = list()
+  timeStampRowNames_list = list()
+  
+  # loop through bundle names
   for (i in 1:nrow(seglist)){
     if(!("emuRsegs" %in% class(seglist)) & !("tbl_df" %in% class(seglist))){
       curUtt = seglist$utts[i]
@@ -299,154 +328,195 @@
                        paste0(splUtt[2], bundle.dir.suffix), 
                        paste0(splUtt[2], ".", trackDef[[1]]$fileExtension))
     
+    # update progressbar
+    if(verbose){
+      utils::setTxtProgressBar(pb, i)
+    }
+    
     ################
     #get data object
-    
-    if(!is.null(onTheFlyFunctionName)){
-      qr = DBI::dbGetQuery(emuDBhandle$connection, paste0("SELECT * ",
-                                                          "FROM bundle ",
-                                                          "WHERE db_uuid = '", emuDBhandle$UUID, "' ",
-                                                          " AND session = '", splUtt[1], "' ",
-                                                          " AND name='", splUtt[2], "'"))
-      funcFormals$listOfFiles = file.path(emuDBhandle$basePath, 
-                                          paste0(qr$session, session.suffix), 
-                                          paste0(qr$name, bundle.dir.suffix), 
-                                          qr$annotates)
-      # only perform calculation if curUtt is not equal to preUtt
-      if(curUtt != prevUtt){
-        curDObj = do.call(onTheFlyFunctionName, funcFormals)
+    if(!is.function(onTheFlyFunction)){
+      if(!is.null(onTheFlyFunctionName)){
+        qr = DBI::dbGetQuery(emuDBhandle$connection, paste0("SELECT * ",
+                                                            "FROM bundle ",
+                                                            "WHERE db_uuid = '", emuDBhandle$UUID, "' ",
+                                                            " AND session = '", splUtt[1], "' ",
+                                                            " AND name='", splUtt[2], "'"))
+        funcFormals$listOfFiles = file.path(emuDBhandle$basePath, 
+                                            paste0(qr$session, session.suffix), 
+                                            paste0(qr$name, bundle.dir.suffix), 
+                                            qr$annotates)
+        # only perform calculation if curUtt is not equal to preUtt
+        if(curUtt != prevUtt){
+          curDObj = do.call(onTheFlyFunctionName, funcFormals)
+        }
+        
+      } else { # if precalculated track read in file
+        curDObj <- wrassp::read.AsspDataObj(fpath)
       }
       
-      if(verbose){
-        utils::setTxtProgressBar(pb, i)
-      }
-    }else{
-      curDObj <- wrassp::read.AsspDataObj(fpath)
-      if(verbose){
-        utils::setTxtProgressBar(pb, i)
-      }
-    }
-    
-    # set origFreq 
-    origFreq <- attr(curDObj, "origFreq")
-    
-    # set curStart+curEnd
-    curStart <- seglist$start[i]
-    if(sum(seglist$end) == 0){
-      curEnd <- seglist$start[i]
-    }else{
-      curEnd <- seglist$end[i]
-    }
-    
-    
-    fSampleRateInMS <- (1 / attr(curDObj, "sampleRate")) * 1000
-    fStartTime <- attr(curDObj, "startTime") * 1000
-    # add one on if event to be able to capture in breakValues 
-    if(sum(seglist$end) == 0){ # if event seglist
-      if(npoints == 1 || is.null(npoints)){
-        timeStampSeq <- seq(fStartTime, curEnd + fSampleRateInMS, fSampleRateInMS)
-      }else{
-        timeStampSeq <- seq(fStartTime, curEnd + fSampleRateInMS * npoints, fSampleRateInMS)
-      }
-    }else{
-      if(npoints == 1 || is.null(npoints)){
-        timeStampSeq <- seq(fStartTime, curEnd, fSampleRateInMS)
-      }else{
-        timeStampSeq <- seq(fStartTime, curEnd + fSampleRateInMS * npoints, fSampleRateInMS)
-      }
-    }
-    
-    ##################################################
-    # search for first element larger than start time
-    breakVal <- -1
-    for (j in 1:length(timeStampSeq)){
-      if (timeStampSeq[j] >= curStart){
-        breakVal <- j
-        break
-      }
-    }
-    # check if breakVal was found
-    if(breakVal == -1){
-      stop("No track samples found belonging to sl_rowIdx: ", i, 
-           " with values: ", paste0(seglist[i,], collapse = " "), 
-           "! This is probably due to a very short SEGMENT that doesn't contain any '", 
-           ssffTrackName, "' values.")
-    }
-    
-    curStartDataIdx <- breakVal
-    curEndDataIdx <- length(timeStampSeq)
-    
-    
-    ################
-    # extract data
-    tmpData <- eval(parse(text = paste("curDObj$", 
-                                       trackDef[[1]]$columnName, 
-                                       sep = "")))
-    
-    #############################################################
-    # set curIndexEnd dependant on if event/segment/cut/npoints
-    if(!is.null(cut) || sum(seglist$end) == 0){
+      # set origFreq 
+      origFreq <- attr(curDObj, "origFreq")
+      
+      # set curStart+curEnd
+      curStart <- seglist$start[i]
       if(sum(seglist$end) == 0){
-        cutTime = curStart
-        curEndDataIdx <- curStartDataIdx
-        curStartDataIdx = curStartDataIdx - 1 # last to elements are relevant -> move start to left
+        curEnd <- seglist$start[i]
       }else{
-        cutTime = curStart + (curEnd - curStart) * cut
+        curEnd <- seglist$end[i]
       }
       
-      sampleTimes = timeStampSeq[curStartDataIdx:curEndDataIdx]
-      closestIdx = which.min(abs(sampleTimes - cutTime))
-      cutTimeSampleIdx = curStartDataIdx + closestIdx - 1
       
-      if(is.null(npoints) || npoints == 1){
-        # reset data idxs
-        curStartDataIdx = curStartDataIdx + closestIdx - 1
-        curEndDataIdx = curStartDataIdx
-        curIndexEnd = curIndexStart
+      fSampleRateInMS <- (1 / attr(curDObj, "sampleRate")) * 1000
+      fStartTime <- attr(curDObj, "startTime") * 1000
+      # add one on if event to be able to capture in breakValues 
+      if(sum(seglist$end) == 0){ # if event seglist
+        if(npoints == 1 || is.null(npoints)){
+          timeStampSeq <- seq(fStartTime, curEnd + fSampleRateInMS, fSampleRateInMS)
+        }else{
+          timeStampSeq <- seq(fStartTime, curEnd + fSampleRateInMS * npoints, fSampleRateInMS)
+        }
       }else{
-        # reset data idx
-        halfNpoints = (npoints - 1) / 2 # -1 removes cutTimeSample
-        curStartDataIdx = cutTimeSampleIdx - floor(halfNpoints)
-        curEndDataIdx = cutTimeSampleIdx + ceiling(halfNpoints)
-        curIndexEnd = curIndexStart + npoints - 1
+        if(npoints == 1 || is.null(npoints)){
+          timeStampSeq <- seq(fStartTime, curEnd, fSampleRateInMS)
+        }else{
+          timeStampSeq <- seq(fStartTime, curEnd + fSampleRateInMS * npoints, fSampleRateInMS)
+        }
       }
       
-    }else{
-      # normal segments
-      curIndexEnd <- curIndexStart + curEndDataIdx - curStartDataIdx
+      ##################################################
+      # search for first element larger than start time
+      breakVal <- -1
+      for (j in 1:length(timeStampSeq)){
+        if (timeStampSeq[j] >= curStart){
+          breakVal <- j
+          break
+        }
+      }
+      # check if breakVal was found
+      if(breakVal == -1){
+        stop("No track samples found belonging to sl_rowIdx: ", i, 
+             " with values: ", paste0(seglist[i,], collapse = " "), 
+             "! This is probably due to a very short SEGMENT that doesn't contain any '", 
+             ssffTrackName, "' values.")
+      }
+      
+      curStartDataIdx <- breakVal
+      curEndDataIdx <- length(timeStampSeq)
+      
+      
+      ################
+      # extract data
+      tmpData <- eval(parse(text = paste("curDObj$", 
+                                         trackDef[[1]]$columnName, 
+                                         sep = "")))
+      
+      #############################################################
+      # set curIndexEnd dependant on if event/segment/cut/npoints
+      if(!is.null(cut) || sum(seglist$end) == 0){
+        if(sum(seglist$end) == 0){
+          cutTime = curStart
+          curEndDataIdx <- curStartDataIdx
+          curStartDataIdx = curStartDataIdx - 1 # last to elements are relevant -> move start to left
+        }else{
+          cutTime = curStart + (curEnd - curStart) * cut
+        }
+        
+        sampleTimes = timeStampSeq[curStartDataIdx:curEndDataIdx]
+        closestIdx = which.min(abs(sampleTimes - cutTime))
+        cutTimeSampleIdx = curStartDataIdx + closestIdx - 1
+        
+        if(is.null(npoints) || npoints == 1){
+          # reset data idxs
+          curStartDataIdx = curStartDataIdx + closestIdx - 1
+          curEndDataIdx = curStartDataIdx
+          curIndexEnd = curIndexStart
+        }else{
+          # reset data idx
+          halfNpoints = (npoints - 1) / 2 # -1 removes cutTimeSample
+          curStartDataIdx = cutTimeSampleIdx - floor(halfNpoints)
+          curEndDataIdx = cutTimeSampleIdx + ceiling(halfNpoints)
+          curIndexEnd = curIndexStart + npoints - 1
+        }
+        
+      }else{
+        # normal segments
+        curIndexEnd <- curIndexStart + curEndDataIdx - curStartDataIdx
+      }
+      # set index and ftime
+      index[i,] <- c(curIndexStart, curIndexEnd)
+      ftime[i,] <- c(timeStampSeq[curStartDataIdx], timeStampSeq[curEndDataIdx])
+      
+      #############################
+      # calculate size of and create new data matrix
+      rowSeq <- seq(timeStampSeq[curStartDataIdx], timeStampSeq[curEndDataIdx], fSampleRateInMS) 
+      curData <- matrix(ncol = ncol(tmpData), nrow = length(rowSeq))
+      
+      # check if it is possible to extract curData 
+      if(curStartDataIdx > 0 && curEndDataIdx <= dim(tmpData)[1]){
+        possibleError_a <- tryCatch(
+          curData[, ] <- tmpData[curStartDataIdx:curEndDataIdx, ],
+          error=function(e) e
+        )
+        # catch error and move on
+        if(inherits(possibleError_a, "error")){
+          warning(paste0("The amount of data extracted doesn't match the \n",
+                         "expected segment length in segment list row ", i, ".\n",
+                         "This can be caused by slight rounding errors in \n",
+                         "sample rates and start times. Adapting to extracted \n",
+                         "sample length."))
+          rowSeq <- timeStampSeq[curStartDataIdx:curEndDataIdx]
+          curData <- matrix(ncol = ncol(tmpData), nrow = length(rowSeq))
+          tmp_len <- length(curStartDataIdx:curEndDataIdx) - length(rowSeq)
+          tmp_range <-  curStartDataIdx:curEndDataIdx
+          possibleError <- tryCatch(
+            curData[, ] <- tmpData[curStartDataIdx:curEndDataIdx, ],
+            error=function(e) e
+          )
+          if(inherits(possibleError, "error")) {
+            stop(paste0("Even after length adaptation an error occured. This shouldn't happen!\n",
+                        "Problematic segment list row: ", i))
+          }
+        }
+        ############# new ##############
+      }else{
+        entry= paste(seglist[i,], collapse = " ")
+        stop('Can not extract data for the ', i, 'th row of the segment list: ', entry, ' start and/or end times out of bounds')
+      }
+      
+      curIndexStart <- curIndexEnd + 1
+      
+      data_list[[i]] = curData
+      timeStampRowNames_list[[i]] = rowSeq
+      
+      prevUtt = curUtt
+    } else {
+      # use custom function
+      customFunctionRes = onTheFlyFunction(file.path(emuDBhandle$basePath, 
+                                                     paste0(splUtt[1], session.suffix), 
+                                                     paste0(splUtt[2], bundle.dir.suffix), 
+                                                     paste0(splUtt[2], ".", DBconfig$mediafileExtension)))
+
+      if(!"frame_time" %in% colnames(customFunctionRes)){
+        stop("The function passed in to onTheFlyFunction didn't return a data.frame with a column called 'frame_time'!")  
+      }
+      
+      customFunctionRes_filtered = customFunctionRes %>% 
+        dplyr::filter(.data$frame_time >= seglist$start[i] & .data$frame_time <= seglist$end[i])
+      
+      # set index and ftime
+      index[i,] <- c(curIndexStart, curIndexStart + nrow(customFunctionRes_filtered) - 1)
+      ftime[i,] <- c(min(customFunctionRes_filtered$frame_time), max(customFunctionRes_filtered$frame_time))
+      
+      # set data_list entry and timeStamp
+      data_list[[i]] = as.matrix(dplyr::select(customFunctionRes_filtered, -.data$frame_time))
+      timeStampRowNames_list[[i]] = customFunctionRes_filtered %>% dplyr::pull(.data$frame_time)
+      
+      # reset start index
+      curIndexStart = curIndexStart + nrow(customFunctionRes_filtered)
     }
-    # set index and ftime
-    index[i,] <- c(curIndexStart, curIndexEnd)
-    ftime[i,] <- c(timeStampSeq[curStartDataIdx], timeStampSeq[curEndDataIdx])
-    
-    #############################
-    # calculate size of and create new data matrix
-    rowSeq <- seq(timeStampSeq[curStartDataIdx], timeStampSeq[curEndDataIdx], fSampleRateInMS) 
-    curData <- matrix(ncol = ncol(tmpData), nrow = length(rowSeq))
-    
-    # check if it is possible to extract curData 
-    if(curStartDataIdx > 0 && curEndDataIdx <= dim(tmpData)[1]){
-      curData[,] <- tmpData[curStartDataIdx:curEndDataIdx,]
-    }else{
-      entry= paste(seglist[i,], collapse = " ")
-      stop('Can not extract data for the ', i, 'th row of the segment list: ', entry, ' start and/or end times out of bounds')
-    }
-    
-    curIndexStart <- curIndexEnd + 1
-    
-    if(!exists("data_list")){
-      # init lists
-      data_list = list()
-      timeStampRowNames_list = list()
-    }
-    
-    data_list[[i]] = curData
-    timeStampRowNames_list[[i]] = rowSeq
-    
-    prevUtt = curUtt
     
   }
-  
   # combind lists to form result
   data = do.call(rbind, data_list)
   timeStampRowNames = unlist(timeStampRowNames_list)
@@ -483,7 +553,7 @@
   if(exists('pb')){
     close(pb)
   }
-  
+
   # convert to emuRtrackdata if resultType is 'emuRtrackdata'
   if(resultType =="emuRtrackdata"){
     resObj = create_emuRtrackdata(seglist, resObj)
