@@ -209,9 +209,9 @@ requery_seq <- function(emuDBhandle,
                           " sl.session, ",
                           " sl.bundle, ",
                           " items_start.item_id AS seq_start_id, ",
-                          " items_end.item_id AS seq_end_id, ", 
+                          " items_end.item_id AS seq_end_id, ",
                           length, " AS seq_len, ",
-                          " sl.level, ",
+                          " sl.attribute AS level, ",
                           " items_start.seq_idx AS seq_start_seq_idx, ",
                           " items_end.seq_idx AS seq_end_seq_idx ",
                           "FROM emursegs_tmp sl ",
@@ -239,7 +239,7 @@ requery_seq <- function(emuDBhandle,
                         " items_start.item_id AS seq_start_id, ",
                         " items_end.item_id AS seq_end_id, ", 
                         length, " AS seq_len, ",
-                        " sl.level, ",
+                        " sl.attribute AS level, ",
                         " items_start.seq_idx AS seq_start_seq_idx, ",
                         " items_end.seq_idx AS seq_end_seq_idx ",
                         "FROM emursegs_tmp AS sl ",
